/**
 * Debounced-mutation-observer
 *
 * ----
 *
 * @author  rileyvel
 * @version 0
 * @licence blessing
 *
 */
import {Mainspring} from 'mainspring-debounce';

export class DebouncedMutationObserver {

    mainspring: Mainspring;
    private observer: MutationObserver;

    /**
     * Creates a new debounced mutation observer.
     * NOTE - monitoring does not start until you call the observe method
     * @param callback
     * @param delay in ms
     */
    constructor(callback: VoidFunction, delay: number) {
        this.mainspring = new Mainspring({ callback, delay });
        this.observer = new MutationObserver(this.rawCallback);
    }

    private rawCallback = () => {
        this.mainspring.arm();
    }

    /**
     * Observe an element. Including its childList, attributes and subtree.
     * @param element
     */
    observe(element: HTMLElement): void {
        this.observer.observe(element, {
            childList: true,
            attributes: true,
            subtree: true,
            characterData: true
        });
    }

    /**
     * Disconnect the observer
     */
    disconnect(): void {
        this.observer.disconnect();
    }

    /**
     * Removes all pending notifications from the MutationObserver's notification queue and
     * returns them in a new Array of MutationRecord objects.
     * Same as MutationObserver's takeRecords.
     */
    takeRecords(): MutationRecord[] {
        return this.observer.takeRecords();
    }

}
